/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * version 2.1 as published by the Free Software Foundation.
 * 
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 */

#ifndef _FVKBD_BASE_H
#define _FVKBD_BASE_H

#include "parser-utility-libxml.h"

G_BEGIN_DECLS

typedef struct _FvkbdUnit FvkbdUnit;
typedef struct _FvkbdUnitClass FvkbdUnitClass;

#define FVKBD_TYPE_UNIT			(fvkbd_unit_get_type())
#define FVKBD_UNIT(obj)			(G_TYPE_CHECK_INSTANCE_CAST((obj), FVKBD_TYPE_UNIT, FvkbdUnit))
#define FVKBD_UNIT_CLASS(klass)		(G_TYPE_CHECK_CLASS_CAST((klass), FVKBD_TYPE_UNIT, FvkbdUnitClass))
#define FVKBD_IS_UNIT(obj)		(G_TYPE_CHECK_INSTANCE_TYPE((obj), FVKBD_TYPE_UNIT))
#define FVKBD_IS_UNIT_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE((klass), FVKBD_TYPE_UNIT))
#define FVKBD_UNIT_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS((obj), FVKBD_TYPE_UNIT, FvkbdUnitClass))

typedef enum _KbdColorType KbdColorType;
enum _KbdColorType {
	KBD_COLOR_TYPE_PANEL_BG = 0,
	KBD_COLOR_TYPE_KEY_BG,
	KBD_COLOR_TYPE_KEY_FG,
	KBD_COLOR_TYPE_KEY_EXTRA_FG,
	KBD_COLOR_TYPE_KEY_POP_BG,
	KBD_COLOR_TYPE_KEY_POP_FG,

	KBD_COLOR_TYPE_NUMBER
};

typedef struct _KbdColor KbdColor;
struct _KbdColor {
	gchar r;
	gchar g;
	gchar b;
	gchar a;
};

struct _FvkbdUnit {
	GObject parent;

	FvkbdUnit *parent_unit;
	gint type;
	gchar *name;
	gfloat x_ratio;
	gfloat y_ratio;
	KbdColor *color[KBD_COLOR_TYPE_NUMBER];
	void *widget;
	void *ui_data;
};

struct _FvkbdUnitClass {
	GObjectClass parent;

	/* Virtual functions */
	gboolean (*parse_xml)		(FvkbdUnit *unit, FvkbdParser *parser);
	gboolean (*build_ui)		(FvkbdUnit *unit, gpointer *widget);
	gboolean (*resize_ui)		(FvkbdUnit *unit, gfloat x_ratio, gfloat y_ratio);
	gboolean (*destroy_ui)		(FvkbdUnit *unit);

	gint (*get_size)		(FvkbdUnit *unit, gint *w, gint *h);
	gint (*set_size)		(FvkbdUnit *unit, gint w, gint h);
	gint (*get_position)		(FvkbdUnit *unit, gint *x, gint *y);
	gint (*set_position)		(FvkbdUnit *unit, gint x, gint y);

	KbdColor * (*get_color)	(FvkbdUnit *unit, KbdColorType type);
	void (*set_color)		(FvkbdUnit *unit, KbdColorType type, KbdColor *color);

	gint (*set_mode)		(FvkbdUnit *unit, gint id);
};

GType fvkbd_unit_get_type (void);

/* definination for fvkbd unit type */
#define UNIT_TYPE_MASK		0xFF000000
#define UNIT_SUBTYPE_MASK	0x00FF0000
#define UNIT_FLAG_MASK		0x0000FFFF

#define UNIT_TYPE_SHIFT		24
#define UNIT_SUBTYPE_SHIFT	16
#define UNIT_FLAG_SHIFT		0

/* type defination for UNIT_TYPE_MASK */
#define UNIT_TYPE_KEYBOARD	0x01
#define UNIT_TYPE_PANEL		0x02
#define UNIT_TYPE_KEY		0x03

/* sub type defination for UNIT_TYPE_PANEL */
#define UNIT_SUBTYPE_PANEL_NORMAL	0x01


typedef struct _UnitParserFunc UnitParserFunc;
struct _UnitParserFunc {
	gchar *node_name;
	gboolean (*parser)			(FvkbdUnit *unit, FvkbdParser *parser, void *data);
};


/* Public Functions */
gboolean fvkbd_unit_parse_xml (FvkbdUnit *unit, FvkbdParser *parser);
gboolean fvkbd_unit_build_ui (FvkbdUnit *unit, gpointer *widget);
gboolean fvkbd_unit_resize_ui (FvkbdUnit *unit, gfloat x_ratio, gfloat y_ratio);
gboolean fvkbd_unit_destroy_ui (FvkbdUnit *unit);

void fvkbd_unit_set_parent (FvkbdUnit *unit, FvkbdUnit *parent);
FvkbdUnit *fvkbd_unit_get_parent (FvkbdUnit *unit);

gint fvkbd_unit_get_size (FvkbdUnit *unit, gint *w, gint *h);
gint fvkbd_unit_set_size (FvkbdUnit *unit, gint w, gint h);
gint fvkbd_unit_get_position (FvkbdUnit *unit, gint *x, gint *y);
gint fvkbd_unit_set_position (FvkbdUnit *unit, gint x, gint y);
KbdColor *fvkbd_unit_get_color (FvkbdUnit *unit, KbdColorType type);
void fvkbd_unit_set_color (FvkbdUnit *unit, KbdColorType type, KbdColor *color);

gint fvkbd_unit_set_mode (FvkbdUnit *unit, gint id);

void fvkbd_unit_set_ui_data (FvkbdUnit *unit, void *data);
void *fvkbd_unit_get_ui_data (FvkbdUnit *unit);

void fvkbd_unit_set_widget (FvkbdUnit *unit, void *data);
void *fvkbd_unit_get_widget (FvkbdUnit *unit);

gint fvkbd_unit_set_ratio (FvkbdUnit *unit, gfloat x_ratio, gfloat y_ratio);
gint fvkbd_unit_get_ratio (FvkbdUnit *unit, gfloat *x_ratio, gfloat *y_ratio);

gint fvkbd_get_unit_type(FvkbdUnit *unit);
void fvkbd_set_unit_type(FvkbdUnit *unit, gint value);
gint fvkbd_get_unit_subtype(FvkbdUnit *unit);
void fvkbd_set_unit_subtype(FvkbdUnit *unit, gint value);
gint fvkbd_get_unit_flag(FvkbdUnit *unit);
void fvkbd_set_unit_flag(FvkbdUnit *unit, gint value);

G_END_DECLS

#endif //_FVKBD_BASE_H