/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * version 2.1 as published by the Free Software Foundation.
 * 
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 */

#ifndef _FVKBD_KEY_H
#define _FVKBD_KEY_H


#include "fvkbd-base.h"

G_BEGIN_DECLS

typedef struct _FvkbdKey FvkbdKey;
typedef struct _FvkbdKeyPrivate FvkbdKeyPrivate;
typedef struct _FvkbdKeyClass FvkbdKeyClass;

#define FVKBD_TYPE_KEY			(fvkbd_key_get_type())
#define FVKBD_KEY(obj)			(G_TYPE_CHECK_INSTANCE_CAST((obj), FVKBD_TYPE_KEY, FvkbdKey))
#define FVKBD_KEY_CLASS(klass)		(G_TYPE_CHECK_CLASS_CAST((klass), FVKBD_TYPE_KEY, FvkbdKeyClass))
#define FVKBD_IS_KEY(obj)		(G_TYPE_CHECK_INSTANCE_TYPE((obj), FVKBD_TYPE_KEY))
#define FVKBD_IS_KEY_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE((klass), FVKBD_TYPE_KEY))
#define FVKBD_KEY_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS((obj), FVKBD_TYPE_KEY, FvkbdKeyClass))

struct _FvkbdKey {
	FvkbdUnit parent;
	FvkbdKeyPrivate *priv;
};

struct _FvkbdKeyClass {
	FvkbdUnitClass parent;
};

GType fvkbd_key_get_type (void);

#define KEY_DEFAULT_WIDTH 50
#define KEY_DEFAULT_HEIGHT 40


typedef enum _KeyActionType KeyActionType;
enum _KeyActionType {
	KEY_ACTION_NONE = 0,
	KEY_ACTION_SYM,
	KEY_ACTION_STRING,
	KEY_ACTION_STRING_GROUP,
	KEY_ACTION_FUNC,
	KEY_ACTION_SCRIPT
};

typedef enum _FvkbdKeyFuncType FvkbdKeyFuncType;
enum _FvkbdKeyFuncType {
	FVKBD_KEY_FUNC_NONE = 0,

	FVKBD_KEY_FUNC_ModShift = 1,
	FVKBD_KEY_FUNC_ModCaps,
	FVKBD_KEY_FUNC_ModControl,
	FVKBD_KEY_FUNC_ModAlt,

	FVKBD_KEY_FUNC_MODE_FIRST = 11,
	FVKBD_KEY_FUNC_MODE_LAST,
	FVKBD_KEY_FUNC_MODE_NEXT,
	FVKBD_KEY_FUNC_MODE_PREV,
	FVKBD_KEY_FUNC_MODE_SELECT,

	FVKBD_KEY_FUNC_EXIT = 21,
	FVKBD_KEY_FUNC_MENU,

	FVKBD_KEY_FUNC_LAST
};

typedef struct _FvkbdKeyFunc FvkbdKeyFunc;
struct _FvkbdKeyFunc {
	FvkbdKeyFuncType type;
	void *data;
};

typedef struct _FvkbdKeyAction FvkbdKeyAction;
struct _FvkbdKeyAction {
	gint mode_id;
	KeyActionType type;
	gchar *disp;
	gchar *img;
	union 
	{
		gchar *string;
		gchar **string_group;
		KeySym sym;
		FvkbdKeyFunc func;
	} u;
};

typedef struct {
	FvkbdKeyFuncType type;
	gchar *name;
} KeyFuncName;

FvkbdUnit *fvkbd_key_new (void);

extern gboolean fvkbd_key_build_ui (FvkbdUnit *unit, gpointer *widget);
extern gboolean fvkbd_key_resize_ui (FvkbdUnit *unit, gfloat x_ratio, gfloat y_ratio);
extern gboolean fvkbd_key_destroy_ui (FvkbdUnit *unit);
extern gint fvkbd_key_set_mode (FvkbdUnit *unit, gint id);

void fvkbd_key_func_handlers (FvkbdKey *self, FvkbdKeyAction *action, void *data);
void fvkbd_key_send_utf8_string (gchar *string);
void fvkbd_key_send_xkeysym (KeySym sym);

gchar *fvkbd_key_get_disp (FvkbdKey *self, gint id);
gchar *fvkbd_key_get_image (FvkbdKey *self, gint id);

FvkbdKeyAction *fvkbd_key_get_current_action (FvkbdKey *self);
FvkbdKeyAction *fvkbd_key_get_action(FvkbdKey *self, gint id);
gchar *fvkbd_key_get_font_family (FvkbdUnit *unit, KbdFontType type);
gchar *fvkbd_key_get_font_weight (FvkbdUnit *unit, KbdFontType type);
gint fvkbd_key_get_font_size (FvkbdUnit *unit, KbdFontType type);


G_END_DECLS

#endif //_FVKBD_KEY_H
