/*
 * fvkbd-keyboard-ui-gtk.c keyboard unit gtk ui for fvkbd  
 *
 * Copyright (C) 2009, Intel Corporation.
 *
 * Author: Raymond Liu <raymond.liu@intel.com>
 * 
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * version 2.1 as published by the Free Software Foundation.
 * 
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 */

#include <gtk/gtk.h>
#include "fvkbd.h"

#include "fvkbd-keyboard.h"
#include "fvkbd-panel.h"
#include "gtk-misc-utility.h"

typedef struct _KeyboardUI KeyboardUI;
struct _KeyboardUI {
	GSList *children;
};

typedef struct _UI_Child UI_Child;
struct _UI_Child {
	FvkbdUnit *unit;
	GtkWidget *widget;
};


gboolean
fvkbd_keyboard_build_ui (FvkbdUnit *unit, gpointer *widget)
{
	GSList *panels;
	FvkbdUnit *panel;
	KeyboardUI *ui_data;
	UI_Child *child = NULL;
	GtkWidget *child_widget = NULL;
	gint i;
	gboolean ret = TRUE;

	panels = fvkbd_keyboard_get_panels(FVKBD_KEYBOARD(unit));
	if (panels == NULL)
		goto done;

	ui_data = g_new0(KeyboardUI, 1);

	for (i = 0; i < g_slist_length(panels); i++) {
		panel = g_slist_nth_data(panels, i);
		if ((fvkbd_get_unit_type(panel) != UNIT_TYPE_PANEL) ||
			(fvkbd_get_unit_subtype(panel) != UNIT_SUBTYPE_PANEL_NORMAL)) {
			continue;
		}

		if ((ret  = fvkbd_unit_build_ui(panel, (gpointer *)&child_widget)) != TRUE)
			goto done;

		child = g_new0(UI_Child, 1);
		child->unit = panel;
		child->widget = child_widget;
		ui_data->children = g_slist_append(ui_data->children, child);
	}

	fvkbd_unit_set_ui_data(unit, ui_data);
done:
	return ret;
}


gboolean
fvkbd_keyboard_resize_ui (FvkbdUnit *unit, gfloat x_ratio, gfloat y_ratio)
{
	fvkbd_unit_set_ratio(unit, x_ratio, y_ratio);
	return TRUE;
}


gboolean
fvkbd_keyboard_destroy_ui (FvkbdUnit *unit)
{
	GSList *list;
	FvkbdUnit *panel;
	KeyboardUI *ui_data;
	gint i;
	gboolean ret = TRUE;

	list= fvkbd_keyboard_get_panels(FVKBD_KEYBOARD(unit));
	if (list == NULL)
		goto done;

	for (i = 0; i < g_slist_length(list); i++) {
		panel = g_slist_nth_data(list, i);
		if ( (fvkbd_get_unit_type(panel) != UNIT_TYPE_PANEL) ||
			(fvkbd_get_unit_subtype(panel) != UNIT_SUBTYPE_PANEL_NORMAL)) {
			continue;
		}

		/* we don't stop upon error, just mark it, and continue the rest works */
		if ( (fvkbd_unit_destroy_ui(panel)) != TRUE)
			ret  = FALSE;
	}

	ui_data = fvkbd_unit_get_ui_data(unit);

	for (list = ui_data->children; list; list = list->next)
		g_free(list->data);

	g_slist_free(ui_data->children);
	ui_data->children = NULL;
	fvkbd_unit_set_ui_data(unit, NULL);
	g_free(ui_data);

done:
	return ret;
}

